@@ -33,6 +33,9 @@
 
 #include <X11/Xatom.h>
 #include <string.h>
+#ifdef HAVE_LIBCANBERRA
+#include <canberra-gtk.h>
+#endif
 
 enum {
   PROP_0,
@@ -170,16 +173,16 @@ meta_workspace_new (MetaScreen *screen)
   meta_screen_foreach_window (screen, maybe_add_to_list, &workspace->mru_list);
 
   workspace->work_areas_invalid = TRUE;
-  workspace->work_area_xinerama = NULL;
+  workspace->work_area_monitor = NULL;
   workspace->work_area_screen.x = 0;
   workspace->work_area_screen.y = 0;
   workspace->work_area_screen.width = 0;
   workspace->work_area_screen.height = 0;
 
   workspace->screen_region = NULL;
-  workspace->xinerama_region = NULL;
+  workspace->monitor_region = NULL;
   workspace->screen_edges = NULL;
-  workspace->xinerama_edges = NULL;
+  workspace->monitor_edges = NULL;
   workspace->list_containing_self = g_list_prepend (NULL, workspace);
 
   workspace->builtin_struts = NULL;
@@ -263,7 +266,7 @@ meta_workspace_remove (MetaWorkspace *workspace)
   workspace->screen->workspaces =
     g_list_remove (workspace->screen->workspaces, workspace);
   
-  g_free (workspace->work_area_xinerama);
+  g_free (workspace->work_area_monitor);
 
   g_list_free (workspace->mru_list);
   g_list_free (workspace->list_containing_self);
@@ -280,12 +283,12 @@ meta_workspace_remove (MetaWorkspace *workspace)
   if (!workspace->work_areas_invalid)
     {
       workspace_free_all_struts (workspace);
-      for (i = 0; i < screen->n_xinerama_infos; i++)
-        meta_rectangle_free_list_and_elements (workspace->xinerama_region[i]);
-      g_free (workspace->xinerama_region);
+      for (i = 0; i < screen->n_monitor_infos; i++)
+        meta_rectangle_free_list_and_elements (workspace->monitor_region[i]);
+      g_free (workspace->monitor_region);
       meta_rectangle_free_list_and_elements (workspace->screen_region);
       meta_rectangle_free_list_and_elements (workspace->screen_edges);
-      meta_rectangle_free_list_and_elements (workspace->xinerama_edges);
+      meta_rectangle_free_list_and_elements (workspace->monitor_edges);
     }
 
   g_object_unref (workspace);
@@ -439,6 +442,69 @@ meta_workspace_queue_calc_showing  (MetaWorkspace *workspace)
     }
 }
 
+static void
+workspace_switch_sound(MetaWorkspace *from,
+                       MetaWorkspace *to)
+{
+#ifdef HAVE_LIBCANBERRA
+  MetaWorkspaceLayout layout;
+  int i, nw, x, y, fi, ti;
+  const char *e;
+
+  nw = meta_screen_get_n_workspaces(from->screen);
+  fi = meta_workspace_index(from);
+  ti = meta_workspace_index(to);
+
+  meta_screen_calc_workspace_layout(from->screen,
+                                    nw,
+                                    fi,
+                                    &layout);
+
+  for (i = 0; i < nw; i++)
+    if (layout.grid[i] == ti)
+      break;
+
+  if (i >= nw)
+    {
+      meta_bug("Failed to find destination workspace in layout\n");
+      goto finish;
+    }
+
+  y = i / layout.cols;
+  x = i % layout.cols;
+
+  /* We priorize horizontal over vertical movements here. The
+     rationale for this is that horizontal movements are probably more
+     interesting for sound effects because speakers are usually
+     positioned on a horizontal and not a vertical axis. i.e. your
+     spatial "Woosh!" effects will easily be able to encode horizontal
+     movement but not such much vertical movement. */
+
+  if (x < layout.current_col)
+    e = "desktop-switch-left";
+  else if (x > layout.current_col)
+    e = "desktop-switch-right";
+  else if (y < layout.current_row)
+    e = "desktop-switch-up";
+  else if (y > layout.current_row)
+    e = "desktop-switch-down";
+  else
+    {
+      meta_bug("Uh, origin and destination workspace at same logic position!\n");
+      goto finish;
+    }
+
+  ca_context_play(ca_gtk_context_get(), 1,
+                  CA_PROP_EVENT_ID, e,
+                  CA_PROP_EVENT_DESCRIPTION, "Desktop switched",
+                  CA_PROP_CANBERRA_CACHE_CONTROL, "permanent",
+                  NULL);
+
+ finish:
+  meta_screen_free_workspace_layout (&layout);
+#endif /* HAVE_LIBCANBERRA */
+}
+
 /**
  * meta_workspace_activate_with_focus:
  * @workspace: a #MetaWorkspace
@@ -478,6 +544,13 @@ meta_workspace_activate_with_focus (MetaWorkspace *workspace,
   if (workspace->screen->active_workspace == workspace)
     return;
 
+  /* Free any cached pointers to the workspaces's edges from
+   * a current resize or move operation */
+  meta_display_cleanup_edges (workspace->screen->display);
+
+  if (workspace->screen->active_workspace)
+    workspace_switch_sound (workspace->screen->active_workspace, workspace);
+
   /* Note that old can be NULL; e.g. when starting up */
   old = workspace->screen->active_workspace;
 
@@ -638,7 +711,7 @@ meta_workspace_update_window_hints (MetaWorkspace *workspace)
  * Gets windows contained on the workspace, including workspace->windows
  * and also sticky windows. Override-redirect windows are not included.
  *
- * Return value: (transfer container): the list of windows.
+ * Return value: (transfer container) (element-type MetaWindow): the list of windows.
  */
 GList*
 meta_workspace_list_windows (MetaWorkspace *workspace)
@@ -713,21 +786,26 @@ meta_workspace_invalidate_work_area (MetaWorkspace *workspace)
               "Invalidating work area for workspace %d\n",
               meta_workspace_index (workspace));
 
-  g_free (workspace->work_area_xinerama);
-  workspace->work_area_xinerama = NULL;
+  /* If we are in the middle of a resize or move operation, we
+   * might have cached pointers to the workspace's edges */
+  if (workspace == workspace->screen->active_workspace)
+    meta_display_cleanup_edges (workspace->screen->display);
+
+  g_free (workspace->work_area_monitor);
+  workspace->work_area_monitor = NULL;
       
   workspace_free_all_struts (workspace);
 
-  for (i = 0; i < workspace->screen->n_xinerama_infos; i++)
-    meta_rectangle_free_list_and_elements (workspace->xinerama_region[i]);
-  g_free (workspace->xinerama_region);
+  for (i = 0; i < workspace->screen->n_monitor_infos; i++)
+    meta_rectangle_free_list_and_elements (workspace->monitor_region[i]);
+  g_free (workspace->monitor_region);
   meta_rectangle_free_list_and_elements (workspace->screen_region);
   meta_rectangle_free_list_and_elements (workspace->screen_edges);
-  meta_rectangle_free_list_and_elements (workspace->xinerama_edges);
-  workspace->xinerama_region = NULL;
+  meta_rectangle_free_list_and_elements (workspace->monitor_edges);
+  workspace->monitor_region = NULL;
   workspace->screen_region = NULL;
   workspace->screen_edges = NULL;
-  workspace->xinerama_edges = NULL;
+  workspace->monitor_edges = NULL;
   
   workspace->work_areas_invalid = TRUE;
 
@@ -780,10 +858,10 @@ ensure_work_areas_validated (MetaWorkspace *workspace)
     return;
 
   g_assert (workspace->all_struts == NULL);
-  g_assert (workspace->xinerama_region == NULL);
+  g_assert (workspace->monitor_region == NULL);
   g_assert (workspace->screen_region == NULL);
   g_assert (workspace->screen_edges == NULL);
-  g_assert (workspace->xinerama_edges == NULL);
+  g_assert (workspace->monitor_edges == NULL);
 
   /* STEP 1: Get the list of struts */
 
@@ -803,18 +881,18 @@ ensure_work_areas_validated (MetaWorkspace *workspace)
   g_list_free (windows);
 
   /* STEP 2: Get the maximal/spanning rects for the onscreen and
-   *         on-single-xinerama regions
+   *         on-single-monitor regions
    */  
-  g_assert (workspace->xinerama_region == NULL);
+  g_assert (workspace->monitor_region == NULL);
   g_assert (workspace->screen_region   == NULL);
 
-  workspace->xinerama_region = g_new (GList*,
-                                      workspace->screen->n_xinerama_infos);
-  for (i = 0; i < workspace->screen->n_xinerama_infos; i++)
+  workspace->monitor_region = g_new (GList*,
+                                      workspace->screen->n_monitor_infos);
+  for (i = 0; i < workspace->screen->n_monitor_infos; i++)
     {
-      workspace->xinerama_region[i] =
+      workspace->monitor_region[i] =
         meta_rectangle_get_minimal_spanning_set_for_region (
-          &workspace->screen->xinerama_infos[i].rect,
+          &workspace->screen->monitor_infos[i].rect,
           workspace->all_struts);
     }
   workspace->screen_region =
@@ -823,7 +901,7 @@ ensure_work_areas_validated (MetaWorkspace *workspace)
       workspace->all_struts);
 
   /* STEP 3: Get the work areas (region-to-maximize-to) for the screen and
-   *         xineramas.
+   *         monitors.
    */
   work_area = workspace->screen->rect;  /* start with the screen */
   if (workspace->screen_region == NULL)
@@ -878,35 +956,35 @@ ensure_work_areas_validated (MetaWorkspace *workspace)
               workspace->work_area_screen.width,
               workspace->work_area_screen.height);    
 
-  /* Now find the work areas for each xinerama */
-  g_free (workspace->work_area_xinerama);
-  workspace->work_area_xinerama = g_new (MetaRectangle,
-                                         workspace->screen->n_xinerama_infos);
+  /* Now find the work areas for each monitor */
+  g_free (workspace->work_area_monitor);
+  workspace->work_area_monitor = g_new (MetaRectangle,
+                                         workspace->screen->n_monitor_infos);
 
-  for (i = 0; i < workspace->screen->n_xinerama_infos; i++)
+  for (i = 0; i < workspace->screen->n_monitor_infos; i++)
     {
-      work_area = workspace->screen->xinerama_infos[i].rect;
+      work_area = workspace->screen->monitor_infos[i].rect;
 
-      if (workspace->xinerama_region[i] == NULL)
+      if (workspace->monitor_region[i] == NULL)
         /* FIXME: constraints.c untested with this, but it might be nice for
          * a screen reader or magnifier.
          */
         work_area = meta_rect (work_area.x, work_area.y, -1, -1);
       else
-        meta_rectangle_clip_to_region (workspace->xinerama_region[i],
+        meta_rectangle_clip_to_region (workspace->monitor_region[i],
                                        FIXED_DIRECTION_NONE,
                                        &work_area);
 
-      workspace->work_area_xinerama[i] = work_area;
+      workspace->work_area_monitor[i] = work_area;
       meta_topic (META_DEBUG_WORKAREA,
                   "Computed work area for workspace %d "
-                  "xinerama %d: %d,%d %d x %d\n",
+                  "monitor %d: %d,%d %d x %d\n",
                   meta_workspace_index (workspace),
                   i,
-                  workspace->work_area_xinerama[i].x,
-                  workspace->work_area_xinerama[i].y,
-                  workspace->work_area_xinerama[i].width,
-                  workspace->work_area_xinerama[i].height);
+                  workspace->work_area_monitor[i].x,
+                  workspace->work_area_monitor[i].y,
+                  workspace->work_area_monitor[i].width,
+                  workspace->work_area_monitor[i].height);
     }
 
   /* STEP 4: Make sure the screen_region is nonempty (separate from step 2
@@ -920,17 +998,17 @@ ensure_work_areas_validated (MetaWorkspace *workspace)
       workspace->screen_region = g_list_prepend (NULL, nonempty_region);
     }
 
-  /* STEP 5: Cache screen and xinerama edges for edge resistance and snapping */
+  /* STEP 5: Cache screen and monitor edges for edge resistance and snapping */
   g_assert (workspace->screen_edges    == NULL);
-  g_assert (workspace->xinerama_edges  == NULL);
+  g_assert (workspace->monitor_edges  == NULL);
   workspace->screen_edges =
     meta_rectangle_find_onscreen_edges (&workspace->screen->rect,
                                         workspace->all_struts);
   tmp = NULL;
-  for (i = 0; i < workspace->screen->n_xinerama_infos; i++)
-    tmp = g_list_prepend (tmp, &workspace->screen->xinerama_infos[i].rect);
-  workspace->xinerama_edges =
-    meta_rectangle_find_nonintersected_xinerama_edges (tmp,
+  for (i = 0; i < workspace->screen->n_monitor_infos; i++)
+    tmp = g_list_prepend (tmp, &workspace->screen->monitor_infos[i].rect);
+  workspace->monitor_edges =
+    meta_rectangle_find_nonintersected_monitor_edges (tmp,
                                                        workspace->all_struts);
   g_list_free (tmp);
 
@@ -950,6 +1028,23 @@ ensure_work_areas_validated (MetaWorkspace *workspace)
   }
 }
 
+static gboolean
+strut_lists_equal (GSList *l,
+                   GSList *m)
+{
+  for (; l && m; l = l->next, m = m->next)
+    {
+      MetaStrut *a = l->data;
+      MetaStrut *b = m->data;
+
+      if (a->side != b->side ||
+          !meta_rectangle_equal (&a->rect, &b->rect))
+        return FALSE;
+    }
+
+  return l == NULL && m == NULL;
+}
+
 /**
  * meta_workspace_set_builtin_struts:
  * @workspace: a #MetaWorkspace
@@ -963,6 +1058,12 @@ void
 meta_workspace_set_builtin_struts (MetaWorkspace *workspace,
                                    GSList        *struts)
 {
+  /* Reordering doesn't actually matter, so we don't catch all
+   * no-impact changes, but this is just a (possibly unnecessary
+   * anyways) optimization */
+  if (strut_lists_equal (struts, workspace->builtin_struts))
+    return;
+
   workspace_free_builtin_struts (workspace);
   workspace->builtin_struts = copy_strut_list (struts);
 
@@ -970,21 +1071,21 @@ meta_workspace_set_builtin_struts (MetaWorkspace *workspace,
 }
 
 void
-meta_workspace_get_work_area_for_xinerama (MetaWorkspace *workspace,
-                                           int            which_xinerama,
-                                           MetaRectangle *area)
+meta_workspace_get_work_area_for_monitor (MetaWorkspace *workspace,
+                                          int            which_monitor,
+                                          MetaRectangle *area)
 {
-  g_assert (which_xinerama >= 0);
+  g_assert (which_monitor >= 0);
 
   ensure_work_areas_validated (workspace);
-  g_assert (which_xinerama < workspace->screen->n_xinerama_infos);
+  g_assert (which_monitor < workspace->screen->n_monitor_infos);
   
-  *area = workspace->work_area_xinerama[which_xinerama];
+  *area = workspace->work_area_monitor[which_monitor];
 }
 
 void
-meta_workspace_get_work_area_all_xineramas (MetaWorkspace *workspace,
-                                            MetaRectangle *area)
+meta_workspace_get_work_area_all_monitors (MetaWorkspace *workspace,
+                                           MetaRectangle *area)
 {
   ensure_work_areas_validated (workspace);
   
@@ -1000,12 +1101,12 @@ meta_workspace_get_onscreen_region (MetaWorkspace *workspace)
 }
 
 GList*
-meta_workspace_get_onxinerama_region (MetaWorkspace *workspace,
-                                      int            which_xinerama)
+meta_workspace_get_onmonitor_region (MetaWorkspace *workspace,
+                                     int            which_monitor)
 {
   ensure_work_areas_validated (workspace);
 
-  return workspace->xinerama_region[which_xinerama];
+  return workspace->monitor_region[which_monitor];
 }
 
 #ifdef WITH_VERBOSE_MODE
@@ -1202,7 +1303,10 @@ focus_ancestor_or_mru_window (MetaWorkspace *workspace,
       MetaWindow *ancestor;
       ancestor = NULL;
       meta_window_foreach_ancestor (not_this_one, record_ancestor, &ancestor);
-      if (ancestor != NULL)
+      if (ancestor != NULL &&
+          (ancestor->on_all_workspaces ||
+           ancestor->workspace == workspace) &&
+          meta_window_showing_on_its_workspace (ancestor))
         {
           meta_topic (META_DEBUG_FOCUS,
                       "Focusing %s, ancestor of %s\n", 
