@@ -152,6 +152,29 @@ meta_debug_topic_get_type (void)
 
   return g_enum_type_id__volatile;
 }
+GType
+meta_later_type_get_type (void)
+{
+  static volatile gsize g_enum_type_id__volatile = 0;
+
+  if (g_once_init_enter (&g_enum_type_id__volatile))
+    {
+      static const GEnumValue values[] = {
+        { META_LATER_RESIZE, "META_LATER_RESIZE", "resize" },
+        { META_LATER_BEFORE_REDRAW, "META_LATER_BEFORE_REDRAW", "before-redraw" },
+        { META_LATER_IDLE, "META_LATER_IDLE", "idle" },
+        { 0, NULL, NULL }
+      };
+      GType g_enum_type_id;
+
+      g_enum_type_id =
+        g_enum_register_static (g_intern_static_string ("MetaLaterType"), values);
+
+      g_once_init_leave (&g_enum_type_id__volatile, g_enum_type_id);
+    }
+
+  return g_enum_type_id__volatile;
+}
 
 /* enumerations from "include/common.h" */
 #include "include/common.h"
@@ -199,6 +222,7 @@ meta_menu_op_get_type (void)
   if (g_once_init_enter (&g_enum_type_id__volatile))
     {
       static const GFlagsValue values[] = {
+        { META_MENU_OP_NONE, "META_MENU_OP_NONE", "none" },
         { META_MENU_OP_DELETE, "META_MENU_OP_DELETE", "delete" },
         { META_MENU_OP_MINIMIZE, "META_MENU_OP_MINIMIZE", "minimize" },
         { META_MENU_OP_UNMAXIMIZE, "META_MENU_OP_UNMAXIMIZE", "unmaximize" },
@@ -1039,6 +1063,7 @@ meta_preference_get_type (void)
         { META_PREF_CURSOR_SIZE, "META_PREF_CURSOR_SIZE", "cursor-size" },
         { META_PREF_COMPOSITING_MANAGER, "META_PREF_COMPOSITING_MANAGER", "compositing-manager" },
         { META_PREF_RESIZE_WITH_RIGHT_BUTTON, "META_PREF_RESIZE_WITH_RIGHT_BUTTON", "resize-with-right-button" },
+        { META_PREF_FORCE_FULLSCREEN, "META_PREF_FORCE_FULLSCREEN", "force-fullscreen" },
         { META_PREF_CLUTTER_PLUGINS, "META_PREF_CLUTTER_PLUGINS", "clutter-plugins" },
         { META_PREF_LIVE_HIDDEN_WINDOWS, "META_PREF_LIVE_HIDDEN_WINDOWS", "live-hidden-windows" },
         { META_PREF_NO_TAB_POPUP, "META_PREF_NO_TAB_POPUP", "no-tab-popup" },
