@@ -86,6 +86,22 @@ static GtkItemFactoryEntry menu_items[] =
   { N_("/Windows/Des_ktop"),      NULL,          NULL,              0, NULL }
 };
 
+static void
+insert_stock_button (GtkWidget          *toolbar,
+                     const gchar        *stock_id,
+                     const gchar        *text)
+{
+  GtkToolItem *button;
+
+  button = gtk_tool_button_new_from_stock (stock_id);
+  gtk_tool_item_set_tooltip_text (button, text);
+  gtk_toolbar_insert (GTK_TOOLBAR (toolbar),
+                      button,
+                      -1); /*-1 means append to end of toolbar*/
+
+  return;
+}
+
 static GtkWidget *
 normal_contents (void)
 {
@@ -131,26 +147,12 @@ normal_contents (void)
    */
   toolbar = gtk_toolbar_new ();
 
-  gtk_toolbar_insert_stock (GTK_TOOLBAR (toolbar),
-                            GTK_STOCK_NEW,
-                            _("Open another one of these windows"),
-                            NULL,
-                            NULL, NULL,
-                            -1);  /* -1 means "append" */
-  
-  gtk_toolbar_insert_stock (GTK_TOOLBAR (toolbar),
-                            GTK_STOCK_OPEN,
-                            _("This is a demo button with an 'open' icon"),
-                            NULL,
-                            NULL, NULL,
-                            -1);  /* -1 means "append" */
-
-  gtk_toolbar_insert_stock (GTK_TOOLBAR (toolbar),
-                            GTK_STOCK_QUIT,
-                            _("This is a demo button with a 'quit' icon"),
-                            NULL,
-                            NULL, NULL,
-                            -1);  /* -1 means "append" */
+  insert_stock_button (toolbar, GTK_STOCK_NEW,
+                       _("Open another one of these windows"));
+  insert_stock_button (toolbar, GTK_STOCK_OPEN,
+                       _("This is a demo button with an 'open' icon"));
+  insert_stock_button (toolbar, GTK_STOCK_QUIT,
+                       _("This is a demo button with a 'quit' icon"));
 
   handlebox = gtk_handle_box_new ();
 
@@ -764,6 +766,7 @@ benchmark_summary (void)
 int
 main (int argc, char **argv)
 {
+  GtkStyle *style;
   GtkWidget *window;
   GtkWidget *collection;
   GError *err;
@@ -840,26 +843,28 @@ main (int argc, char **argv)
                     G_CALLBACK (gtk_main_quit), NULL);
 
   gtk_widget_realize (window);
-  g_assert (window->style);
-  g_assert (window->style->font_desc);
+  style = gtk_widget_get_style (window);
+
+  g_assert (style);
+  g_assert (style->font_desc);
   
   notebook = gtk_notebook_new ();
   gtk_container_add (GTK_CONTAINER (window), notebook);
 
   collection = preview_collection (FONT_SIZE_NORMAL,
-                                   window->style->font_desc);
+                                   style->font_desc);
   gtk_notebook_append_page (GTK_NOTEBOOK (notebook),
                             collection,
                             gtk_label_new (_("Normal Title Font")));
   
   collection = preview_collection (FONT_SIZE_SMALL,
-                                   window->style->font_desc);
+                                   style->font_desc);
   gtk_notebook_append_page (GTK_NOTEBOOK (notebook),
                             collection,
                             gtk_label_new (_("Small Title Font")));
   
   collection = preview_collection (FONT_SIZE_LARGE,
-                                   window->style->font_desc);
+                                   style->font_desc);
   gtk_notebook_append_page (GTK_NOTEBOOK (notebook),
                             collection,
                             gtk_label_new (_("Large Title Font")));
@@ -910,7 +915,7 @@ get_flags (GtkWidget *widget)
 static int
 get_text_height (GtkWidget *widget)
 {
-  return meta_pango_font_desc_get_text_height (widget->style->font_desc,
+  return meta_pango_font_desc_get_text_height (gtk_widget_get_style (widget)->font_desc,
                                                gtk_widget_get_pango_context (widget));
 }
 
@@ -991,7 +996,7 @@ run_theme_benchmark (void)
       /* Creating the pixmap in the loop is right, since
        * GDK does the same with its double buffering.
        */
-      pixmap = gdk_pixmap_new (widget->window,
+      pixmap = gdk_pixmap_new (gtk_widget_get_window (widget),
                                client_width + left_width + right_width,
                                client_height + top_height + bottom_height,
                                -1);
